import unittest

from office.image import add_watermark
from office.tools import url2ip, weather



class TestTools(unittest.TestCase):

    def test_weather(self):
        weather()

    def test_url2ip(self):
        url2ip('www.python-office.com')

    def test_image_watermark(self):
        add_watermark(file=r'.\目录结构.jpg',mark='程序员晚枫')



"""
功能：消除图片背景
"""

from PIL import Image

def _hex_to_rgb(hex):
    """
    十六进制转RGB
    """
    if hex[0] != '#' or len(hex) != 7:   
        print('注意：十六进制格式颜色错误，请输入7位以\'#\'开头的字符串\n')
        return None
    else:
        r = int('0x' + hex[1:3], 16)
        g = int('0x' + hex[3:5], 16)
        b = int('0x' + hex[5:7], 16)
        return (r, g, b)

def eliminate_bc(src_img_path, save_img_path, margin=30, bc_color=None):
    """
    将图片的背景变成透明色
    参数：
        src_img_path: string, 原始图片存储路径
        margin: int, 和背景颜色的差异值
        bc_color, string or tuple 背景颜色值（十六进制或RGB值）
    """
    img = Image.open(src_img_path)
    width, height = img.size
    
    # 获取背景颜色的RGB值
    if bc_color:
        # 给定背景色
        if isinstance(bc_color, str):
            r, g, b = _hex_to_rgb(bc_color)
        else:
            r, g, b = bc_color
    else:
        # 未给定背景色，拾取图片左上角颜色作为背景色
        pix = img.load()
        if src_img_path.endswith('.jpg'):
            r, g, b = pix[int(width / 20), int(height / 20)]
        elif src_img_path.endswith('.png'):
            r, g, b, _ = pix[int(width / 20), int(height / 20)]

    img = img.convert("RGBA")
    datas = img.getdata()
    newData = list()

    # 背景填充零透明度
    for item in datas:
        if (item[0] >= max(r - margin, 0) and item[0] <= min(r + margin, 255)) \
            and (item[1] >= max(g - margin, 0) and item[1] <= min(g + margin, 255)) \
            and (item[2] >= max(b - margin, 0) and item[2] <= min(b + margin, 255)):
            newData.append((255, 255, 255, 0))
        else:
            newData.append(item)    
    img.putdata(newData)

    # 保存新图片
    img.save(save_img_path, "PNG")

if __name__ == '__main__':

    # 未设定背景颜色        
    eliminate_bc('test.jpg', 'a.png')

    # 设定背景颜色
    # eliminate_bc('test.jpg', 'a.png', bc_color=(255, 255, 255))
    # eliminate_bc('test.jpg', 'a.png', bc_color='#FFFFFF')



import os
from fpdf import FPDF
from office.lib.pdf import add_watermark_service
import pikepdf
from PyPDF2 import PdfFileReader, PdfFileWriter
from pdf2docx import Converter


class MainPDF():

    def add_watermark(self):
        pdf_file_in = input("请输入需要添加水印的文件位置：")  # 需要添加水印的文件
        Watermark_Str = input("请输入需要添加的水印内容：")
        print('=' * 20)
        print('正在按要求，给你的PDF文件添加水印，请让程序飞一会儿~')
        print('=' * 20)
        pdf_file_mark = 'watermark.pdf'  # 水印文件
        add_watermark_service.create_watermark(str(Watermark_Str))
        pdf_file_out = '添加了水印的文件.pdf'  # 添加PDF水印后的文件
        add_watermark_service.pdf_add_watermark(pdf_file_in, pdf_file_mark, pdf_file_out)
        print("水印添加结束，请打开电脑上的这个位置，查看结果文件：{path}".format(path=os.getcwd()))

    def file2pdf(self, file_type, path, res_pdf='file2pdf.pdf'):
        if file_type == 'txt':
            pdf = FPDF()
            pdf.add_page()  # Add a page
            pdf.set_font("Arial", size=15)  # set style and size of font
            f = open(path, "r")  # open the text file in read mode
            # insert the texts in pdf
            for x in f:
                pdf.cell(50, 5, txt=x, ln=1, align='C')
            # pdf.output("path where you want to store pdf file\\file_name.pdf")
            pdf.output(res_pdf)

    def pdf2docx(self, file_path):
        try:
            pdf_name = file_path.split('.')[0]
            word_name = pdf_name + '.docx'
            cv = Converter(file_path)
            cv.convert(word_name)
            cv.close()
        except:
            print('这个文件有问题~！')

    # 合并pdf
    def merge2pdf(self, one_by_one, output):
        """
        @Author & Date  : CoderWanFeng 2022/5/16 23:33
        @Desc  : merge_pdfs(paths=['开篇词.pdf', '中国元宇宙白皮书 (送审稿).pdf'], output='merge.pdf')
        """
        pdf_writer = PdfFileWriter()

        for path in one_by_one:
            pdf_reader = PdfFileReader(path)
            for page in range(pdf_reader.getNumPages()):
                # 把每张PDF页面加入到这个可读取对象中
                pdf_writer.addPage(pdf_reader.getPage(page))

        # 把这个已合并了的PDF文档存储起来
        with open(output, 'wb') as out:
            pdf_writer.write(out)

    # PDF加密
    def encrypt4pdf(self, path, password, res_pdf='encrypt.pdf'):
        """
        @Author & Date  : CoderWanFeng 2022/5/9 18:27
        @Desc  : path: 存放文件的路径
                password: 你的密码
                res_pdf: 结果文件的名称 ，可以为空，默认是：encrypt.pdf
        """
        pdf = pikepdf.open(path)
        pdf.save(res_pdf, encryption=pikepdf.Encryption(owner=password, user=password, R=4))
        pdf.close()

    # PDF解密
    def decrypt4pdf(self, path, password, res_pdf='decrypt.pdf'):
        pdf = pikepdf.open(path, password=password)
        pdf.save(res_pdf)
        pdf.close()



import os
import time

from office.lib.ppt.ppt2pdf_service import ppt2pdf_single


class MainPPT():

    def ppt2pdf(self, path):
        """
        @Author & Date  : CoderWanFeng 2022/5/9 23:34
        @Desc  : path:存放ppt的路径，必须写绝对路径~！
        """
        # 列出指定目录的内容
        filenames = os.listdir(path)
        # for循环依次访问指定目录的所有文件名
        for filename in filenames:
            # 判断文件的类型，对所有的ppt文件进行处理(ppt文件以ppt或者pptx结尾的)
            if filename.endswith('ppt') or filename.endswith('pptx'):
                # print(filename)           # PPT素材1.pptx -> PPT素材1.pdf
                # 将filename以.进行分割，返回2个信息，文件的名称和文件的后缀名
                base, ext = filename.split('.')  # base=PPT素材1 ext=pdf
                new_name = base + '.pdf'  # PPT素材1.pdf
                # ppt文件的完整位置: C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pptx
                filename = path + '/' + filename
                # pdf文件的完整位置: C:/Users/Administrator/Desktop/PPT办公自动化/ppt/PPT素材1.pdf
                output_filename = path + '/' + new_name
                # 将ppt转成pdf文件
                ppt2pdf_single(filename, output_filename)
                time.sleep(3)



from office.lib.utils.except_utils import except_dec

from office.core.ToolsType import MainTools

mainTools = MainTools()

@except_dec()
def transtools(to_lang, content):
    mainTools.transtools(to_lang, content)

@except_dec()
def qrcodetools(url):
    mainTools.qrcodetools(url)

@except_dec()
def passwordtools(len=8):
    mainTools.passwordtools(len)

@except_dec()
def weather():
    mainTools.weather()


# 通过url，获取ip地址
@except_dec()
def url2ip(url):
    mainTools.url2ip(url)



"""
python-office Command Line Interface
====================================

```script
python -m office --help
```
"""
from typer import Typer

app = Typer(name="python-office CLI")


#
# @app.command()
# def ppt2pdf(dir_path: str):
#     """ppt导出为PDF，需要Microsoft Office环境"""
#     try:
#         from office.ppt import ppt2pdf
#     except ImportError:
#         from office.ppt import ppt2pdf
#     return ppt2pdf(dir_path)
#

@app.command()
def docx2pdf(dir_path: str, suffix="docx"):
    """docx转pdf"""
    from office.word import docx2pdf
    Typer.launch("http://www.python4office.cn/wechat-group/")
    return docx2pdf(dir_path, suffix)


@app.command()
def hello():
    print(666)


#
#
# @app.command()
# def pdf_encrypt(input_path: str, password: str, output_path: str = None):
#     """加密PDF 输入路径 密码 [输出路径(默认与输入相同)]"""
#     try:
#         from .pdf import encrypt4pdf
#     except ImportError:
#         from pdf import encrypt4pdf
#     return encrypt4pdf(input_path, password, output_path or input_path)
#
#
# @app.command()
# def pdf_decrypt(input_path: str, password: str, output_path: str = None):
#     """解密PDF 输入路径 密码 [输出路径(默认与输入相同)]"""
#     try:
#         from .pdf import decrypt4pdf
#     except ImportError:
#         from pdf import decrypt4pdf
#     return decrypt4pdf(input_path, password, output_path or input_path)
#
#
# @app.command()
# def pdf_merge(paths: list[str], output_path: str):
#     """合并PDF 若干pdf路径 最后一个参数为合并后的PDF的路径"""
#     try:
#         from .pdf import merge2pdf
#     except ImportError:
#         from pdf import merge2pdf
#     return merge2pdf(paths, output_path)
#
#
# @app.command()
# def watermark(path: str, mark: str, out_dir="output", color="#8B8B1B", size=30, opacity=0.15, space=75, angle=30):
#     """图片水印 文件(夹)路径 水印文本 [输出文件夹] [颜色] [字号] [透明度] [字间距] [倾角]"""
#     try:
#         from .image import add_watermark
#     except ImportError:
#         from image import add_watermark
#     return add_watermark(path, mark, out_dir, color, size, opacity, space, angle)
#
#
# @app.command()
# def txt2pdf(txt_path: str, pdf_path: str):
#     """创建纯文本PDF 输入路径 输出路径"""
#     try:
#         from .pdf import txt2pdf
#     except ImportError:
#         from pdf import txt2pdf
#     return txt2pdf(txt_path, pdf_path)
#
#
# @app.command()
# def rename(path: str, old: str, new=""):
#     """批量修改文件名 目录路径 子字符串 [新字符串]"""
#     try:
#         from .file import replace4filename
#     except ImportError:
#         from file import replace4filename
#     return replace4filename(path, old, new)
#
#
# @app.command()
# def fake(path="./fake2excel.xlsx", rows=1, columns=("name",), language='zh_CN'):
#     """生成假数据Excel文件 [导出文件路径] [行数] [列名列表] [语种]"""
#     try:
#         from .excel import fake2excel
#     except ImportError:
#         from excel import fake2excel
#     return fake2excel(columns, rows, language, path)
#
#
# @app.command()
# def pwd(length: int = 8):
#     """生成随机密码"""
#     try:
#         from .tools import passwordtools
#     except ImportError:
#         from tools import passwordtools
#     return passwordtools(length)
#
#
# @app.command()
# def extract_audio(input_path: str, output_path: str = None):
#     """从视频中提取音频"""
#     try:
#         from .video import video2mp3
#     except ImportError:
#         from video import video2mp3
#     return video2mp3(input_path,
#                      (input_path[:input_path.rindex(".")] or output_path.removesuffix(".mp3")) + ".mp3")
#

if __name__ == '__main__':
    app()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: 文件.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 文件 的自动化操作
#############################################
from office.lib.utils.except_utils import except_dec

from office.core.FileType import MainFile
from office.core.SearchByContentType import MainSearchByContent

mainFile = MainFile()
mainSearchByContent = MainSearchByContent()

# todo：输入文件路径
@except_dec()
def replace4filename(path, del_content, replace_content=None):
    mainFile.replace4filename(path, del_content, replace_content)

# todo：输入文件路径
@except_dec()
def search_by_content(search_path, content):  # 定义 search() 函数，传入 "path" 文件路径， "target" 要查找的目标文件
    mainSearchByContent.search_by_content(search_path, content)



# -*- coding: utf-8 -*-
import reportlab
from PyPDF2 import PdfFileWriter, PdfFileReader
from reportlab.pdfgen import canvas
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase.pdfmetrics import registerFont

def create_watermark(content):
    """创建PDF水印模板
    """
    # 创建一个PDF文件来作为一个水印文件
    c = canvas.Canvas('watermark.pdf')
    reportlab.pdfbase.pdfmetrics.registerFont(
        reportlab.pdfbase.ttfonts.TTFont('simfang', 'C:/Windows/Fonts/simfang.ttf'))
    c.setFont('simfang', 20)
    c.saveState()
    c.translate(305, 505)
    c.rotate(45)
    c.drawCentredString(0, 0, content)
    c.restoreState()
    c.save()
    pdf_watermark = PdfFileReader('watermark.pdf')
    return pdf_watermark


def pdf_add_watermark(pdf_file_in, pdf_file_mark, pdf_file_out):
    # print(pdf_file_out)
    pdf_output = PdfFileWriter()
    input_stream = open(pdf_file_in, 'rb')
    pdf_input = PdfFileReader(input_stream, strict=False)
    # 获取PDF文件的页数
    if pdf_input.getIsEncrypted():
        print("文件已被加密")
        PDF_Passwd = input("请输入PDF密码：")
        # 尝试用空密码解密
        try:
            pdf_input.decrypt(PDF_Passwd)
        except Exception:
            print(f"尝试用密码{PDF_Passwd}解密失败.")
            return False
    pageNum = pdf_input.getNumPages()
    # 读入水印pdf文件
    # print(pdf_file_mark)
    mark_stream = open(pdf_file_mark, mode='rb')
    pdf_watermark = PdfFileReader(mark_stream, strict=False)
    # 给每一页打水印
    for i in range(pageNum):
        page = pdf_input.getPage(i)
        page.mergePage(pdf_watermark.getPage(0))
        page.compressContentStreams()  # 压缩内容
        pdf_output.addPage(page)
    pdf_output.write(open(pdf_file_out, 'wb'))




from office.api import word
from office.api import pdf
from office.api import image
from office.api import file
from office.api import video
from office.api import tools
from office.api import ppt
from office.api import excel

print('=' * 30)
print('【python-office库】，功能持续更新中')
print( '全部功能 - 说明书 -：https://www.python-office.com/')
print('使用有问题 or 提交你的功能需求 or 参与项目开发')
print('请+【开发者微信】：http://t.cn/A6XVQXAk')
print('=' * 30)



import time


def time_count_dec(func):
    def wrapper(*args, **kwargs):
        t1 = time.time()
        res = func(*args, **kwargs)
        t2 = time.time()
        print(func.__name__ + "执行耗时" + str(t2 - t1))
        return res

    return wrapper



import os
from win32com.client import constants, gencache


class MainWord():

    def file2pdf(self, path, docxSuffix=".docx"):
        wordFiles = []
        # 如果不存在，则不做处理
        if not os.path.exists(path):
            print("path does not exist path = " + path)
            return
        # 判断是否是文件
        elif os.path.isfile(path):
            print("path file type is file " + path)
            wordFiles.append(path)
        # 如果是目录，则遍历目录下面的文件
        elif os.path.isdir(path):
            print(os.listdir(path))
            # 填充路径，补充完整路径
            if not path.endswith("/") or not path.endswith("\\"):
                path = path + "/"
            for file in os.listdir(path):
                if file.endswith(docxSuffix):
                    wordFiles.append(path + file)
        print(wordFiles)
        for file in wordFiles:
            filepath = os.path.abspath(file)
            index = filepath.rindex('.')
            pdfPath = os.path.abspath(filepath[:index] + '.pdf')
            print(pdfPath)
            self.createpdf(filepath, pdfPath)


    def createpdf(self,wordPath, pdfPath):
        word = gencache.EnsureDispatch('Word.Application')
        doc = word.Documents.Open(wordPath, ReadOnly=1)
        # 转换方法
        doc.ExportAsFixedFormat(pdfPath, constants.wdExportFormatPDF)
        word.Quit()



from faker import Faker
import pandas as pd
from alive_progress import alive_bar
from office.lib.utils import pandas_mem
import os
from pathlib import Path
from openpyxl import load_workbook
import warnings

# 忽略waring警告
warnings.filterwarnings("ignore")


class MainExcel():

    def fake2excel(self, columns, rows, path, language):
        """
        @Author & Date  : CoderWanFeng 2022/5/13 0:12
        @Desc  : columns:list，每列的数据名称，默认是名称
                rows：多少行，默认是1
                language：什么语言，可以填english，默认是中文
                path：输出excel的位置，有默认值
        """
        # 可以选择英语
        if language.lower() == 'english':
            language = 'en_US'
        else:
            language = 'zh_CN'
        # 开始造数
        fake = Faker(language)
        excel_dict = {}
        with alive_bar(len(columns) * rows) as bar:
            for column in columns:
                excel_dict[column] = list()
                # excel_dict[column] = map(lambda x: eval('fake.{func}()'.format(func=x)), [column] * rows) # 使用map，会报错
                while len(excel_dict[column]) < rows:
                    excel_dict[column].append(eval('fake.{func}()'.format(func=column)))
                    bar()
            # 用pandas，将模拟数据，写进excel里面
            writer = pd.ExcelWriter(path)
            data = pd.DataFrame(excel_dict)
            data = pandas_mem.reduce_pandas_mem_usage(data)
            data.to_excel(writer, index=False)
            # writer.save()
            writer.close()

    def merge2excel(self, dir_path, output_file, xlsxSuffix=".xlsx"):
        """
        :param dir_path:
        :param output_file:
        :return:
        """
        if not output_file.endswith(xlsxSuffix):
            raise Exception(f'您自定义的输出文件名，不是以{xlsxSuffix}结尾的')
        file_path_dict = self.getfile(dir_path)  # excel文件所在的文件夹
        try:
            writer = pd.ExcelWriter(output_file)  # 合并后的excel名称
        except PermissionError:
            raise Exception(f'小可爱，你的输出文件，是不是上次打开了没关闭呀？这是你自己指定的输出文件名称：{output_file}')
        for file, path in file_path_dict.items():
            if file.endswith("xlsx"):
                df = pd.read_excel(path)
            if file.endswith("csv"):
                df = pd.read_csv(path)
            df.to_excel(writer, sheet_name=file.split('.')[0], index=False)
        print(f'您指定的Excel文件已经合并完毕，合并后的文件名是{output_file}')
        writer.save()

    def getfile(self, dirpath):
        path = Path(dirpath)
        file_path_dict = {}
        for root, dirs, files in os.walk(dirpath):
            for file in files:
                if file.endswith("xlsx") or file.endswith("csv"):
                    file_path_dict[file] = (path / file)
        return file_path_dict

    def sheet2excel(self, file_path):
        # 先读取一次文件，获取sheet表的名称

        origin_excel = load_workbook(filename=file_path)  # 读取原excel文件
        origin_sheet_names = origin_excel.sheetnames  # 获取sheet的名称
        print(f'一共有{len(origin_sheet_names)}个sheet，名称分别为：{origin_sheet_names}')
        print('拆分开始')

        if len(origin_sheet_names) > 1:  # 如果sheetnames小于1，报错：该文件不需要拆分

            for j in range(len(origin_sheet_names)):

                wb = load_workbook(filename=file_path)  # 再读取一次文件，由于每次删除后需要保存一次，所以不能与上一次一样
                sheet = wb[origin_sheet_names[j]]
                wb.copy_worksheet(sheet)

                new_filename = origin_sheet_names[j] + '.xlsx'  # 新建一个sheet命名的excel文件

                for i in range(len(origin_sheet_names)):
                    sheet1 = wb[origin_sheet_names[i]]
                    wb.remove(sheet1)

                wb.save(filename=new_filename)

                # 由于使用copy_worksheet后，sheet表名有copy字段，这里做个调整

                new = load_workbook(filename=new_filename)
                news = new.active
                news.title = origin_sheet_names[j]
                new.save(filename=new_filename)
            print('拆分结束')
        else:
            raise Exception(f"你的文件只有一个sheet，难道还要拆分吗？我做不到啊~~~，你的文件名{file_path}")



# -*- coding: utf-8 -*-

from office.core.PDFType import MainPDF
from office.lib.utils.except_utils import except_dec

mainPDF = MainPDF()


# 给pdf加水印
@except_dec()
def add_watermark() -> None:
    mainPDF.add_watermark()


# txt转pdf
@except_dec()
def txt2pdf(path: str, res_pdf='txt2pdf.pdf'):
    mainPDF.file2pdf(path, res_pdf)


# PDF加密
@except_dec()
def encrypt4pdf(path, password, res_pdf='encrypt.pdf'):
    mainPDF.encrypt4pdf(path, password, res_pdf)


# PDF解密
@except_dec()
def decrypt4pdf(path, password, res_pdf='decrypt.pdf'):
    mainPDF.decrypt4pdf(path, password, res_pdf)


# 合并pdf
@except_dec()
def merge2pdf(one_by_one, output):
    mainPDF.merge2pdf(one_by_one, output)


# todo：输入文件路径
@except_dec()
def pdf2docx(file_path):
    mainPDF.pdf2docx(file_path)



import moviepy.editor as mp


class MainVideo():

    # 从视频里提取音频
    def video2mp3(self, path, mp3_name):
        """
        :param path: 视频文件的路径
        :param mp3_name: mp3的名字，可以为空
        :return:
        """
        # specify the mp4 file here(mention the file path if it is in different directory)
        clip = mp.VideoFileClip(path)
        if mp3_name:
            clip.audio.write_audiofile(str(mp3_name) + '.mp3')
        else:
            # specify the name for mp3 extracted
            clip.audio.write_audiofile('Audio.mp3')
            # you will notice mp3 file will be created at the specified location.




"""
图片添加水印，参考：
"""

import math
import os

from PIL import Image, ImageFont, ImageDraw, ImageEnhance, ImageChops

TTF_FONT = os.path.dirname(__file__) + "/font/msyh.ttc"


def crop_image(im):
    '''裁剪图片边缘空白'''
    bg = Image.new(mode='RGBA', size=im.size)
    bbox = ImageChops.difference(im, bg).getbbox()
    if bbox:
        return im.crop(bbox)
    return im


def set_opacity(im, opacity):
    '''设置水印透明度'''
    assert 0 <= opacity <= 1
    alpha = im.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im


def get_mark_img(text, color="#8B8B1B", size=30, opacity=0.15):
    """生成水印图片"""
    width = len(text) * size
    mark = Image.new(mode='RGBA', size=(width, size + 20))
    draw_table = ImageDraw.Draw(im=mark)
    draw_table.text(xy=(0, 0),
                    text=text,
                    fill=color,
                    font=ImageFont.truetype(TTF_FONT, size=size))
    del draw_table
    # 裁剪空白
    mark = crop_image(mark)
    # 透明度
    set_opacity(mark, opacity)
    return mark


def im_add_mark(im, text, color="#8B8B1B", size=30, opacity=0.15, space=75, angle=30):
    """给图片对象添加水印"""
    # 获取水印图片对象
    mark = get_mark_img(text, color, size, opacity)
    # 将水印图片扩展并旋转生成水印大图
    w, h = im.size
    c = int(math.sqrt(w ** 2 + h ** 2))
    mark2 = Image.new(mode='RGBA', size=(c, c))
    y, idx = 0, 0
    mark_w, mark_h = mark.size
    while y < c:
        x = -int((mark_w + space) * 0.5 * idx)
        idx = (idx + 1) % 2
        while x < c:
            mark2.paste(mark, (x, y))
            x = x + mark_w + space
        y = y + mark_h + space
    # 将水印大图旋转一定角度
    mark2 = mark2.rotate(angle)
    # 在原图上添加水印大图
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    im.paste(mark2, (int((w - c) / 2), int((h - c) / 2)),  # 坐标
             mask=mark2.split()[3])
    return im


def add_mark2file(imageFile, text, out="output", color="#8B8B1B", size=30, opacity=0.15, space=75, angle=30):
    '''
    添加水印，然后保存图片
    '''
    name = os.path.basename(imageFile)
    new_name = os.path.join(out, name)
    try:
        im = Image.open(imageFile)
        image = im_add_mark(im, text, color, size, opacity, space, angle)
        if not os.path.exists(out):
            os.mkdir(out)
        if os.path.splitext(new_name)[1] != '.png':
            image = image.convert('RGB')
        image.save(new_name)
        print(new_name, "保存成功。")
    except Exception as e:
        print(new_name, "保存失败。错误信息：", e)



from datetime import datetime
# import traceback
from functools import wraps


# 统一的异常输出
def except_dec(msg='异常原因'):
    # msg用于自定义函数的提示信息
    def except_execute(func):
        @wraps(func)
        def execept_print(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                print('=' * 30)

                print('糟糕，你的程序出现了异常')
                print(
                    f'>>>异常时间：\t{datetime.now()}\n>>>异常函数：\t{func.__name__}\n>>>{msg}：\t{e}')

                print('别慌，你的异常也许【群友也遇到过】 → http://t.cn/A65MiFvH')
                print('当然，也可以免费【加入星球，向我提问】 → http://t.cn/A6qeZpVt')


                print('=' * 30)

                # print(f'{sign}{traceback.format_exc()}{sign}')

        return execept_print

    return except_execute



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: word.py
# Author: 程序员晚枫
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关word的自动化操作
#############################################
from office.lib.utils.except_utils import except_dec

from office.core.WordType import MainWord

# 创建对象
mainWord = MainWord()


# 1、文件的批量转换
# 自己指定路径，
# 为了适配wps不能转换doc的问题，这里限定：只能转换docx
@except_dec()
def docx2pdf(path):
    mainWord.file2pdf(path)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: ppt.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 ppt 的自动化操作
#############################################
from office.lib.utils.except_utils import except_dec

from office.core.PPTType import MainPPT

mainPPT = MainPPT()


# todo：输入文件路径
@except_dec()
def ppt2pdf(path: str):
    mainPPT.ppt2pdf(path)



import os
from alive_progress import alive_bar


class MainFile():

    def replace4filename(self, path, del_content, replace_content):
        """
        :param path: 需要替换的文件夹路径
        :param del_content: 需要删除/替换的内容
        :param replace_content: 替换后的内容，可以不填 = 直接删除
        :return:
        """
        # 获取该目录下所有文件，存入列表中；不包含子文件夹
        fileList = os.listdir(path)
        work_count = 0
        with alive_bar(len(fileList)) as bar:
            for old_file_name in fileList:  # 依次读取该路径下的文件名
                bar()  # 进度条
                if del_content in old_file_name:

                    if replace_content:
                        new_file_name = old_file_name.replace(del_content, replace_content)
                    else:
                        new_file_name = old_file_name.replace(del_content, '')
                    os.rename(path + os.sep + old_file_name, path + os.sep + new_file_name)
                    work_count = work_count + 1
        print("当前目录下，共有{}个文件/文件夹，本次运行共进行了{}个文件/文件夹的重命名".format(len(fileList), work_count))




import requests
import re

#天气爬虫
def weather_spider(url,headers):
    response = requests.get(url,headers)
    content = response.content.decode('utf-8')
    pat_weather = re.compile('<input type="hidden" id="hidden_title" value="(.*?)" />')
    pat_up_time = re.compile('<input type="hidden" id="fc_24h_internal_update_time" value="(.*?)"/>')
    weather = pat_weather.findall(content)
    up_time = pat_up_time.findall(content)
    print(weather[0])
    print('更新时间：',up_time[0])
    ask_ok = input('是否深入查看（Y/N）：')
    if ask_ok == 'Y' or ask_ok == 'y':
        pat_more_weather = re.compile('<li class="li. hot".*?\n<i></i>.<span>(.*?)</span>\n<em>(.*?)</em>\n<p>(.*?)</p>.*?\n</li>',re.S)
        more_weather = pat_more_weather.findall(content)
        for item in more_weather:
            if item[1] != '减肥指数':
                print(item[1],':',item[0],',',item[2])
            else:
                print(item[1],':',item[2])





from datetime import datetime
# import traceback
from functools import wraps


# 统一的异常输出
def except_dec(msg='异常原因'):
    # msg用于自定义函数的提示信息
    def except_execute(func):
        @wraps(func)
        def execept_print(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                sign = '=' * 60 + '\n'
                print(f'{sign}>>>异常时间：\t{datetime.now()}\n>>>异常函数：\t{func.__name__}\n>>>{msg}：\t{e}\n>>>异常反馈：\thttp://www.python4office.cn/wechat-group/')

                print(f'{sign}')
                # print(f'{sign}{traceback.format_exc()}{sign}')

        return execept_print

    return except_execute

@except_dec()
def lig(a = 5,b = 0):
    print(a/b)
lig()


#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: excel.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 excel 的自动化操作
#############################################
from office.core.ExcelType import MainExcel
from office.lib.utils.except_utils import except_dec

mainExcel = MainExcel()


# todo:输出文件路径
@except_dec()
def fake2excel(columns=['name'], rows=1, path='./fake2excel.xlsx', language='zh_CN'):
    mainExcel.fake2excel(columns, rows, path, language)


# 多个excel，合并到一个excel的不同sheet中
@except_dec()
def merge2excel(dir_path, output_file='merge2excel.xlsx'):
    """
    :param dir_path:
    :param output_file:
    :return:
    """
    mainExcel.merge2excel(dir_path, output_file)


# 同一个excel里的不同sheet，拆分为不同的excel文件
@except_dec()
def sheet2excel(file_path):
    mainExcel.sheet2excel(file_path)



from translate import Translator
import qrcode
import string
import random
import socket

from office.lib.tools.weather_city_code import WEATHER_CITY_CODE_DIC
from office.lib.tools.weather_service import weather_spider
# from utils.tools.weather_city_code import WEATHER_CITY_CODE_DIC


class MainTools():

    def transtools(self, to_lang, content):
        # specifying the language
        translator = Translator(to_lang)
        # typing the message
        translation = translator.translate(content)
        # print the translated message
        print(translation)

    def qrcodetools(self, url):
        # Creating object
        # version: defines size of image from integer(1 to 40), box_size = size of each box in pixels, border = thickness of the border.
        qr = qrcode.QRCode(version=1, box_size=10, border=5)
        # add_date :  pass the input text
        qr.add_data(url)
        # converting into image
        qr.make(fit=True)
        # specify the foreground and background color for the img
        img = qr.make_image(fill='black', back_color='white')
        # store the image
        img.save('qrcode_img.png')

    def passwordtools(self, len):
        """
        @Author & Date  : CoderWanFeng 2022/5/9 11:54
        @Desc  : 随机密码生成器，默认是8位
        @Return  ：
        """
        chars = string.digits + string.ascii_letters
        return ''.join(random.sample(chars * 10, len))

    def weather(self):
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/604.4.7 (KHTML, like Gecko) Version/11.0.2 Safari/604.4.7'
        }
        while (True):
            try:
                cityName = input('请输入城市名称(按q/Q键退出)：')
                if cityName == 'q' or cityName == 'Q':
                    break
                cityCode = WEATHER_CITY_CODE_DIC[cityName]  # 得到城市代码
                url = 'http://www.weather.com.cn/weather1d/%d.shtml' % cityCode  # 得到城市天气网址
                weather_spider(url, headers)
            except:
                print('未查到%s城市，请重新输入：' % cityName)


    # 通过url，获取ip地址
    def url2ip(self,url):
        socket_list = socket.getaddrinfo(url, None, 0, socket.SOCK_STREAM)
        ip_info = socket_list[0][4][0]
        print('【{}】 这个网址对应的IP地址是：{}'.format(url, ip_info))